"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
exports.__esModule = true;
var util_1 = require("../util");
var ContentTypeResolverBuilder = /** @class */ (function () {
    function ContentTypeResolverBuilder(dataSource, contentType) {
        this.dataSource = dataSource;
        this.contentType = contentType;
    }
    ContentTypeResolverBuilder.prototype.build = function () {
        var _this = this;
        var contentType = this.contentType;
        var fields = {};
        var typeName = util_1.idToName(this.contentType.sys.id);
        contentType.fields.forEach(function (f) {
            return fields[f.id] = _this.fieldResolver(f);
        });
        var withTypeName = {};
        withTypeName[typeName] = fields;
        return withTypeName;
    };
    ContentTypeResolverBuilder.prototype.fieldResolver = function (field) {
        if (field.omitted) {
            return function () { return undefined; };
        }
        switch (field.type) {
            case 'Symbol':
            case 'Text':
            case 'Date':
            case 'Integer':
            case 'Number':
            case 'Boolean':
                return function (entry) { return entry.fields[field.id]; };
            case 'Location':
                return function (entry) { return (__assign({ __typename: 'Location' }, entry.fields[field.id])); };
            case 'Link':
                return this.linkResolver(field);
            case 'Array':
                if (field.items && field.items.type == 'Link') {
                    return this.collectionResolver(field);
                }
                else {
                    // Simple list - get the items
                    return function (entry) { return entry.fields[field.id]; };
                }
            default:
                return function (entry) { return (__assign({ __typename: 'JSON' }, entry.fields[field.id])); };
        }
    };
    ContentTypeResolverBuilder.prototype.linkResolver = function (field) {
        var _this = this;
        var dataSource = this.dataSource;
        return function (entry) { return __awaiter(_this, void 0, void 0, function () {
            var link, resolved, resolved;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        link = entry.fields[field.id];
                        if (!link) {
                            return [2 /*return*/, null];
                        }
                        if (!link.sys || link.sys.type != 'Link') {
                            throw new Error("Value in field " + field.id + " is not a link! (was '" + link + "')");
                        }
                        if (!(field.linkType == 'Asset')) return [3 /*break*/, 2];
                        return [4 /*yield*/, dataSource.getAsset(link.sys.id)];
                    case 1:
                        resolved = _a.sent();
                        return [2 /*return*/, __assign({ __typename: 'Asset' }, resolved)];
                    case 2: return [4 /*yield*/, dataSource.getEntry(link.sys.id)];
                    case 3:
                        resolved = _a.sent();
                        return [2 /*return*/, __assign({ __typename: util_1.idToName(entry.sys.contentType.sys.id) }, resolved)];
                }
            });
        }); };
    };
    ContentTypeResolverBuilder.prototype.collectionResolver = function (field) {
        var _this = this;
        var dataSource = this.dataSource;
        var linkType = field.items.linkType;
        return function (entry, args) { return __awaiter(_this, void 0, void 0, function () {
            var links, linkIDs, collection_1, collection;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        links = entry.fields[field.id];
                        linkIDs = links.map(function (link) {
                            if (!link) {
                                return null;
                            }
                            if (!link.sys || link.sys.type != 'Link') {
                                throw new Error("Value in field " + field.id + " is not a link! (was '" + link + "')");
                            }
                            return link.sys.id;
                        }).filter(util_1.present);
                        if (!(linkType == 'Asset')) return [3 /*break*/, 2];
                        return [4 /*yield*/, dataSource.getAssets(__assign(__assign({}, args), { 'sys.id[in]': linkIDs }))];
                    case 1:
                        collection_1 = _a.sent();
                        return [2 /*return*/, {
                                skip: collection_1.skip,
                                limit: collection_1.limit,
                                total: collection_1.total,
                                items: collection_1.items.map(function (entry) { return (__assign({ __typename: 'Asset' }, entry)); })
                            }];
                    case 2: return [4 /*yield*/, dataSource.getEntries(__assign(__assign({}, args), { 'sys.id[in]': linkIDs }))];
                    case 3:
                        collection = _a.sent();
                        return [2 /*return*/, {
                                skip: collection.skip,
                                limit: collection.limit,
                                total: collection.total,
                                items: collection.items.map(function (entry) { return (__assign({ __typename: util_1.idToName(entry.sys.contentType.sys.id) }, entry)); })
                            }];
                }
            });
        }); };
    };
    return ContentTypeResolverBuilder;
}());
exports["default"] = ContentTypeResolverBuilder;
//# sourceMappingURL=content-type-resolver-builder.js.map