"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var inflection_1 = __importDefault(require("inflection"));
var graphql_type_json_1 = __importDefault(require("graphql-type-json"));
var graphql_1 = require("graphql");
var types_1 = require("../types");
var util_1 = require("../util");
var ContentTypeWriter = /** @class */ (function () {
    function ContentTypeWriter(contentType, contentTypeMap, collectionTypeMap) {
        var _this = this;
        this.contentType = contentType;
        this.contentTypeMap = contentTypeMap;
        this.collectionTypeMap = collectionTypeMap;
        this.write = function () {
            var contentType = _this.contentType;
            var type = new graphql_1.GraphQLObjectType({
                name: _this.className,
                interfaces: [types_1.Entry],
                fields: function () {
                    var fields = {};
                    contentType.fields.forEach(function (f) {
                        return _this.writeField(f, fields);
                    });
                    return fields;
                }
            });
            _this.contentTypeMap.set(_this.className, type);
            var collection = new graphql_1.GraphQLObjectType({
                name: _this.className + "Collection",
                fields: {
                    skip: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLInt) },
                    limit: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLInt) },
                    total: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLInt) },
                    items: { type: new graphql_1.GraphQLNonNull(new graphql_1.GraphQLList(type)) }
                }
            });
            _this.collectionTypeMap.set(_this.className, collection);
            return {
                type: type,
                collection: collection
            };
        };
        this.linkedTypes = [];
        var name = util_1.idToName(contentType.sys.id);
        this.className = name;
    }
    ContentTypeWriter.prototype.writeField = function (field, fields) {
        var nullable = field.omitted || (!field.required);
        var type = this.writeFieldType(field);
        fields[field.id] = {
            type: nullable ? type : new graphql_1.GraphQLNonNull(type)
        };
    };
    ContentTypeWriter.prototype.writeFieldType = function (field) {
        if (field.omitted) {
            return types_1.GraphQLNever;
        }
        switch (field.type) {
            case 'Symbol':
            case 'Text':
            case 'Date':
                return this.writePotentialUnionType(field) || graphql_1.GraphQLString;
            case 'Integer':
                return this.writePotentialUnionType(field) || graphql_1.GraphQLInt;
            case 'Number':
                return this.writePotentialUnionType(field) || graphql_1.GraphQLFloat;
            case 'Boolean':
                return graphql_1.GraphQLBoolean;
            case 'Location':
                return types_1.GraphQLLocation;
            case 'Link':
                if (field.linkType == 'Asset') {
                    return types_1.Asset;
                }
                else {
                    return this.resolveLinkContentType(field);
                }
            case 'Array':
                var itemType = this.writeFieldType(Object.assign({ id: field.id }, field.items));
                if (field.items.type == 'Link') {
                    if (itemType == types_1.Asset) {
                        return types_1.AssetCollection;
                    }
                    else if (itemType == types_1.Entry) {
                        return types_1.EntryCollection;
                    }
                    return this.writeLinkCollectionType(itemType);
                }
                else {
                    return new graphql_1.GraphQLList(itemType);
                }
            default:
                return graphql_type_json_1["default"];
        }
    };
    ContentTypeWriter.prototype.resolveLinkContentType = function (field) {
        var _a;
        var _this = this;
        var validation = field.validations &&
            field.validations.filter(util_1.isLinkContentTypeValidation).find(function (v) {
                return v.linkContentType.length > 0;
            });
        if (!validation) {
            return types_1.Entry;
        }
        (_a = this.linkedTypes).push.apply(_a, __spreadArray([], __read(validation.linkContentType)));
        if (validation.linkContentType.length == 1) {
            var name_1 = util_1.idToName(validation.linkContentType[0]);
            var resolved = this.contentTypeMap.get(name_1);
            if (!resolved) {
                throw new Error("Could not resolve content type '" + name_1 + "'");
            }
            return resolved;
        }
        var unionName = unionTypeDefName(this.contentType.sys.id, field);
        return new graphql_1.GraphQLUnionType({
            name: unionName,
            types: function () { return validation.linkContentType.map(function (val) {
                var name = util_1.idToName(val);
                var resolved = _this.contentTypeMap.get(name);
                if (!resolved) {
                    throw new Error("Could not resolve content type '" + name + "'");
                }
                return resolved;
            }); }
        });
    };
    ContentTypeWriter.prototype.writeLinkCollectionType = function (itemType) {
        if (this.collectionTypeMap.has(itemType.name)) {
            var collection = this.collectionTypeMap.get(itemType.name);
            return collection;
        }
        else {
            var collection = new graphql_1.GraphQLObjectType({
                name: itemType.name + "Collection",
                fields: {
                    skip: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLInt) },
                    limit: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLInt) },
                    total: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLInt) },
                    items: { type: new graphql_1.GraphQLNonNull(new graphql_1.GraphQLList(itemType)) }
                }
            });
            this.collectionTypeMap.set(itemType.name, collection);
            return collection;
        }
    };
    ContentTypeWriter.prototype.writePotentialUnionType = function (field) {
        if (!field.validations) {
            return null;
        }
        var validation = field.validations.find(function (v) { return v["in"] && v["in"].length > 0; });
        if (validation) {
            var name_2 = unionTypeDefName(this.contentType.sys.id, field);
            return new graphql_1.GraphQLEnumType({
                name: name_2,
                values: validation["in"].reduce(function (map, val) {
                    map[val.toString()] = {
                        value: val
                    };
                    return map;
                }, {})
            });
        }
    };
    return ContentTypeWriter;
}());
exports["default"] = ContentTypeWriter;
function unionTypeDefName(contentType, field) {
    return "" + util_1.idToName(contentType) + inflection_1["default"].singularize(util_1.idToName(field.id));
}
//# sourceMappingURL=content-type-writer.js.map