"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var graphql_1 = require("graphql");
var inflection_1 = __importDefault(require("inflection"));
var content_type_writer_1 = __importDefault(require("./content-type-writer"));
var SchemaBuilder = /** @class */ (function () {
    function SchemaBuilder(options) {
        var opts = Object.assign({
            filename: 'contentful-schema.json',
            logger: console
        }, options);
        this.options = opts;
    }
    SchemaBuilder.prototype.build = function () {
        var contentfulSchema;
        if ('contentTypes' in this.options) {
            contentfulSchema = { contentTypes: this.options.contentTypes };
        }
        else {
            contentfulSchema = require(this.options.filename);
        }
        var contentTypeMap = new Map();
        var helperTypeMap = new Map();
        var graphQLTypes = contentfulSchema.contentTypes.map(function (ct) {
            return new content_type_writer_1["default"](ct, contentTypeMap, helperTypeMap).write();
        });
        var Query = new graphql_1.GraphQLObjectType({
            name: 'Query',
            fields: graphQLTypes.reduce(function (fields, _a) {
                var type = _a.type, collection = _a.collection;
                var queryFieldName = inflection_1["default"].camelize(type.name, true);
                fields[queryFieldName] = {
                    type: type,
                    args: {
                        id: { type: graphql_1.GraphQLString }
                    }
                };
                fields[queryFieldName + "Collection"] = {
                    type: collection,
                    args: {
                        skip: { type: graphql_1.GraphQLString },
                        limit: { type: graphql_1.GraphQLString }
                    }
                };
                return fields;
            }, {})
        });
        return new graphql_1.GraphQLSchema({
            query: Query
        });
    };
    return SchemaBuilder;
}());
exports["default"] = SchemaBuilder;
//# sourceMappingURL=index.js.map