/// <reference types="node" />
import type FS from 'fs';
export interface SchemaDownloaderOptions {
    /**
     * The name of the schema file.  Defaults to './contentful-schema.json'.
     */
    filename: string;
    /**
     * Contentful space ID.  Defaults to process.env.CONTENTFUL_SPACE_ID
     */
    space: string;
    /**
     * Contentful environment.
     *
     * Defaults to process.env.CONTENTFUL_ENVIRONMENT or 'master'
     */
    environment: string;
    /**
     * Contentful Management Access Token.
     *
     * Defaults to process.env.CONTENTFUL_MANAGEMENT_TOKEN
     */
    managementToken: string;
    logger: {
        debug: Console['debug'];
    };
}
export declare class SchemaDownloader {
    private readonly options;
    private readonly client;
    private readonly semaphore;
    private readonly fs;
    constructor(options?: Partial<SchemaDownloaderOptions>, fs?: typeof FS);
    downloadSchema(): Promise<void>;
    private getSchemaFromSpace;
    private requestLogger;
    private responseLogger;
}
