import type { ContentfulClientApi } from "contentful";
import { SyncItem } from "../dataSource";
export interface Syncable {
    getToken(): string | undefined | null | Promise<string | undefined | null>;
    setToken(token: string): void | Promise<void>;
    index(syncItem: SyncItem): void | Promise<void>;
}
export declare class SyncEngine {
    private readonly dataSource;
    private readonly client;
    constructor(dataSource: Syncable, client: Pick<ContentfulClientApi, 'sync'>);
    sync(): Promise<void>;
}
/**
 * Enhances a dataSource with a Contentful client to keep it up to date via the
 * Sync API.
 * @param dataSource The data source to store data e.g. InMemoryDataSource, PostgresDataSource
 * @param client The Contentful client that allows us to sync data to the data source
 * @returns The same data source enhanced with the sync() method
 */
export declare function withSync<DataSource extends Syncable>(dataSource: DataSource, client: Pick<ContentfulClientApi, 'sync'>): DataSource & Pick<SyncEngine, 'sync'>;
