"use strict";
exports.__esModule = true;
exports.AssetCollection = exports.assetFieldResolver = exports.Asset = exports.EntryCollection = exports.Entry = exports.ContentfulMetadata = exports.ContentfulTag = exports.Sys = exports.GraphQLLocation = exports.GraphQLNever = void 0;
var graphql_1 = require("graphql");
exports.GraphQLNever = new graphql_1.GraphQLScalarType({
    name: 'Never'
});
exports.GraphQLLocation = new graphql_1.GraphQLObjectType({
    name: 'Location',
    fields: {
        lat: { type: graphql_1.GraphQLFloat },
        lon: { type: graphql_1.GraphQLFloat }
    }
});
exports.Sys = new graphql_1.GraphQLObjectType({
    name: 'Sys',
    fields: {
        id: { type: graphql_1.GraphQLString },
        spaceId: { type: graphql_1.GraphQLString },
        environmentId: { type: graphql_1.GraphQLString }
    }
});
exports.ContentfulTag = new graphql_1.GraphQLObjectType({
    name: 'ContentfulTag',
    fields: {
        id: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLString) },
        name: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLString) }
    }
});
exports.ContentfulMetadata = new graphql_1.GraphQLObjectType({
    name: 'ContentfulMetadata',
    fields: {
        tags: { type: new graphql_1.GraphQLNonNull(new graphql_1.GraphQLList(exports.ContentfulTag)) }
    }
});
exports.Entry = new graphql_1.GraphQLInterfaceType({
    name: 'Entry',
    fields: {
        sys: { type: exports.Sys },
        contentfulMetadata: { type: exports.ContentfulMetadata }
    }
});
exports.EntryCollection = new graphql_1.GraphQLObjectType({
    name: 'EntryCollection',
    fields: {
        skip: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLInt) },
        limit: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLInt) },
        total: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLInt) },
        items: { type: new graphql_1.GraphQLNonNull(new graphql_1.GraphQLList(exports.Entry)) }
    }
});
exports.Asset = new graphql_1.GraphQLObjectType({
    name: 'Asset',
    fields: {
        sys: { type: exports.Sys },
        contentfulMetadata: { type: exports.ContentfulMetadata },
        // linkedFrom: AssetLinkingCollections
        title: { type: graphql_1.GraphQLString },
        description: { type: graphql_1.GraphQLString },
        contentType: { type: graphql_1.GraphQLString },
        fileName: { type: graphql_1.GraphQLString },
        url: { type: graphql_1.GraphQLString },
        size: { type: graphql_1.GraphQLInt },
        width: { type: graphql_1.GraphQLInt },
        height: { type: graphql_1.GraphQLInt }
    }
});
function assetFieldResolver() {
    return {
        title: function (asset) { return asset.fields.title; },
        description: function (asset) { return asset.fields.description; },
        contentType: function (asset) { var _a; return (_a = asset.fields.file) === null || _a === void 0 ? void 0 : _a.contentType; },
        fileName: function (asset) { var _a; return (_a = asset.fields.file) === null || _a === void 0 ? void 0 : _a.fileName; },
        url: function (asset) { var _a; return (_a = asset.fields.file) === null || _a === void 0 ? void 0 : _a.url; },
        size: function (asset) { var _a, _b; return (_b = (_a = asset.fields.file) === null || _a === void 0 ? void 0 : _a.details) === null || _b === void 0 ? void 0 : _b.size; },
        width: function (asset) { var _a, _b, _c; return (_c = (_b = (_a = asset.fields.file) === null || _a === void 0 ? void 0 : _a.details) === null || _b === void 0 ? void 0 : _b.image) === null || _c === void 0 ? void 0 : _c.width; },
        height: function (asset) { var _a, _b, _c; return (_c = (_b = (_a = asset.fields.file) === null || _a === void 0 ? void 0 : _a.details) === null || _b === void 0 ? void 0 : _b.image) === null || _c === void 0 ? void 0 : _c.height; }
    };
}
exports.assetFieldResolver = assetFieldResolver;
exports.AssetCollection = new graphql_1.GraphQLObjectType({
    name: 'AssetCollection',
    fields: {
        skip: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLInt) },
        limit: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLInt) },
        total: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLInt) },
        items: { type: new graphql_1.GraphQLNonNull(new graphql_1.GraphQLList(exports.Asset)) }
    }
});
//# sourceMappingURL=types.js.map